#include <Foundation/Foundation.h>
#include <AppKit/NSImage.h>
#include "FileRing.h"


NSString * PhMakePath(NSString * dir, NSString * path);
NSString * PhMakeAbsPath(NSString * path);

void PhPrintArray(NSArray * arr);


@implementation FileRing

- (id) init
{
    _dir = nil;
    _files = nil;
    _pos = 0;
    return self;
}

- (void) dealloc
{
    RELEASE(_dir);
    RELEASE(_files);
}

- (id) initFromPath: (NSString *)path withTypes: (NSArray *) allowedTypes
{
    // NSLog(@"[FileRing initFromPath: %s\n", [path cString]);
    // PhPrintArray(allowedTypes);
        
    NSString * dir = nil;
    BOOL isDir;
    NSFileManager * fmgr = [NSFileManager defaultManager];

    //NSLog(@"current directory: %s\n", [[fmgr currentDirectoryPath] cString]);

    if ([fmgr fileExistsAtPath:path isDirectory: &isDir])
    {
        dir = (isDir) ? path : [path stringByDeletingLastPathComponent];
    }
    else
    {
        NSLog(@"[FileRing initFromPath: invalid path '%s'\n", [path cString]);
        return [self init];
    }

    _dir = [[NSString alloc] initWithString: PhMakeAbsPath(dir)];

    
    // Create files ring

    _files = [[NSMutableArray alloc] init];


    NSEnumerator * content
        = [[fmgr directoryContentsAtPath: _dir] objectEnumerator];
    NSString * name = nil;
    NSString * absPath = nil;
    while (name = [content nextObject])
    {
        NSDictionary * dict =
            [fmgr fileAttributesAtPath: PhMakePath(_dir, name)
                  traverseLink: YES];
        
        if (dict && [[dict fileType] isEqualToString: NSFileTypeRegular])
        {
            if ( [allowedTypes containsObject: [name pathExtension]] )
            {
                // NSLog(@"FileRing: adding %s\n", [name cString]);
                
                // Add the name to the ring
                [_files addObject: name];
            }
        }
    }

    // Sort
    [_files sortUsingSelector: @selector(caseInsensitiveCompare:)];
    
    // Find position of the 'path' into '_files'
    _pos = 0;
    if (isDir == NO)
    {
        _pos = [_files indexOfObject: [path lastPathComponent]];
        if (_pos == NSNotFound)
            _pos = 0;
    }

    // PhPrintArray(_files);
    // NSLog(@"    count=%d, pos=%d\n", [_files count], _pos);
    
    return self;
}

- (NSString *) directory
{
    return _dir;
}

- (NSString *) currentName
{
    NSString * res = ([_files count] > 0) ? [_files objectAtIndex: _pos] : nil;
    return res;
}

- (NSString *) currentFile
{
    return PhMakePath(_dir, [self currentName]);
}

- (NSString *) nextFile
{
    NSAssert(_files, @"nextImage: '_files' is not initialized");
    NSAssert2(_pos < [_files count],
              @"pos=%d >= count=%d", _pos, [_files count]);

    ++_pos;
    if (_pos >= [_files count])
        _pos = 0;

    return [self currentFile];
}

- (NSString *) prevFile
{
    NSAssert(_files, @"nextImage: '_files' is not initialized");
    NSAssert2(_pos < [_files count],
              @"pos=%d >= count=%d", _pos, [_files count]);

    if (_pos == 0)
    {
        if ([_files count] > 0)
            _pos = [_files count] - 1;
    }
    else
    {
        --_pos;
    }

    return [self currentFile];
}

@end

NSString * PhMakePath(NSString * dir, NSString * path)
{
    if ( [path isAbsolutePath] == NO )
        path = [dir stringByAppendingPathComponent: path];

    return path;
}

NSString * PhMakeAbsPath(NSString * path)
{
    if ( [path isAbsolutePath] == NO )
    {
        NSFileManager * fmgr = [NSFileManager defaultManager];
        path =
            [[fmgr currentDirectoryPath] stringByAppendingPathComponent: path];
    }

    return path;
}

void PhPrintArray(NSArray * arr)
{
    NSEnumerator * typesEnum = [arr objectEnumerator];
    id type;
    while (type = [typesEnum nextObject])
        NSLog(@"type=%s\n", [type cString]);
}
